function [Pos,Years,Data_raw,Info_raw] = get_sscweb_data(Object)
%GET_SSCWEB_DATA Downloads an parses data from SSCWeb at NSSDC
%
%   Object_list = GET_SSCWEB_DATA returns list of available Objects.  
%
%   [Pos,Years] = GET_SSCWEB_DATA(Object) returns all available position
%   data for Object in D.  Pos spans first day of Years(1) to the last
%   day of Years(2).  Flag value for missing data is 99999.
%              
%   [Pos,Years,Data_raw,Data_name] = GET_SSCWEB_DATA(Object) returns 
%   raw data extracted from HTML file along with the column names.
%    
%   This .m program uses the command line program "curl" to download the
%   HTML page and the .m program perlop.m to parse the HTML file.
%
%   For information on options, see get_sscweb_data.htm, which is a copy of
%   http://nssdc.gsfc.nasa.gov/space/helios/heli.html
%


% R.S. Weigel, 01/12/2004 

more off;

if (is_octave)
  error('Not yet Octave-compatable');
end
  

INFO = {'COMET_GRIGGSKJ'       '1992'  '001'  '1993'  '001'  '01';...
	'COMET_HMP'            '1996'  '001'  '1996'  '062'  '02';...
	'COMET_HALEBOPP'       '1997'  '001'  '1997'  '365'  '03';...
	'EARTH'                '1965'  '001'  '2015'  '365'  '04';...
	'GALILEO'              '1989'  '299'  '2015'  '365'  '05';...
	'GIOTTO'               '1992'  '001'  '1992'  '366'  '06';...
	'HELIOS_1'             '1974'  '345'  '1985'  '365'  '07';...
	'HELIOS_2'             '1976'  '001'  '1985'  '365'  '08';...
	'ICE'                  '1984'  '001'  '1999'  '365'  '09';...
	'PIONEER_10'           '1972'  '063'  '2005'  '365'  '10';...
	'PIONEER_11'           '1973'  '096'  '1999'  '365'  '11';...
	'SAKIGAKE'             '1985'  '021'  '1997'  '365'  '12';...
	'SUISEI'               '1989'  '001'  '1999'  '365'  '13';...
	'ULYSSES'              '1990'  '292'  '2015'  '343'  '14';...
	'VENUS_PVO'            '1978'  '001'  '1999'  '365'  '15';...
	'VOYAGER_1'            '1977'  '252'  '2015'  '343'  '16';...
	'VOYAGER_2'            '1977'  '237'  '2015'  '343'  '17';...
	'COMET_BORRELLY'       '2000'  '001'  '2002'  '365'  '18';...
	'COMET_WILD_2'         '2002'  '001'  '2004'  '366'  '19';...
	'COMET_HYAKUTAKE'      '1995'  '244'  '1996'  '366'  '20';...
	'PHOBOS_2'             '1988'  '195'  '1989'  '030'  '21';...
	'COMET_HALLEY'         '1984'  '001'  '1987'  '365'  '22';...
	'COMET_GACOBINIZINNER' '1984'  '001'  '1986'  '365'  '23';...
	'CASSINI'              '1997'  '289'  '2004'  '183'  '24'};

name = strvcat(INFO{:,1});

if (nargin == 0)
  Info_raw = INFO;
  return;
end

obj_no = strmatch(Object,name);
if (length(obj_no) > 1)
  error('Object name is not unique.  Include more characters in input string.');
end
if (isempty(obj_no))
  error('Object not available');
end

key         = strvcat(INFO{:,6});
days_start  = strvcat(INFO{:,3});
days_end    = strvcat(INFO{:,5});	      
years_start = strvcat(INFO{:,2});	      
years_end   = strvcat(INFO{:,4});	      

object      = key(obj_no,:);
coordinate  = 1; % Solar Ecliptic system

start_year  = str2num(years_start(obj_no,:));
stop_year   = str2num(years_end(obj_no,:));

Ny          = (stop_year-start_year+1);

% There is a limit on the number of returned values of 1461.
% So we will grab just one year at a time.
for i = 1:Ny

  the_year = start_year+i-1;

  start_day = '001';
  stop_day  = '365';

  if (is_leap_year(the_year))
    stop_day  = '366';
  end

  if  (i == Ny)
    stop_day   = days_end(obj_no,:);
  end
  if  (i == 1)
    start_day  = days_start(obj_no,:);
  end

  post_string1 = sprintf('spacecraft/object=%s&coordinate=%d&',...
			 object,coordinate);
  post_string2 = sprintf('start_year=%d&start_day=%s&stop_year=%d&',...
			 the_year,start_day,the_year);
  post_string3 = sprintf('stop_day=%s&resolution=001&equinox=2',stop_day);
  
  post_string  = [post_string1,post_string2,post_string3];

  temp_file    = sprintf('temp_%s',Object);

  % Use curl to grab the post the data and pipe the data to a file
  % named temp.
  URL = sprintf('http://lewes.gsfc.nasa.gov/cgi-bin/models/helic');
  com = sprintf('curl -d ''%s'' "%s" > %s',post_string,URL,temp_file);
  fprintf('\n\nget_sscweb_data is calling system with command:\n\n%s\n\n',com);
  system(com);
  
  REGEXP = '=~ m/YEAR/'; % Match line with the string YEAR
  [I,A]  = perlop(temp_file,REGEXP,[],[],'file');
  
  REGEXP        = '=~ s/<code>(.*[A-Z])<\/code>.*/$1/'; % Grab the labels
  [I,Info_raw]  = perlop(A,REGEXP);
    
  REGEXP = '=~ s/<|>|code|\/|br|pre|[A-Z]|_//g'; % Remove HTML and strings
  [I,B]  = perlop(A,REGEXP);

  B      = reshape(B,5,length(B)/5)';
    
  Dc{i} = B;

end

Data_raw = cat(1,Dc{:});

Years(1)  = Data_raw(1,1);
Years(2)  = Data_raw(end,1);
Nd        = number_days(Years(1),Years(2));
temp      = repmat(99999,Nd,3);
DN        = day_number( ymd(Data_raw(:,2),Data_raw(:,1),'day') );
DN        = DN - DN(1) + 1;
Pos(DN,:) = Data_raw(:,3:5); 


rm(temp_file);
