/*  arx.c evaluates an AutoRegressive eXogenous (ARX) function */
/*  

    With gcc, compile using: gcc -lm -o arx.o arx.c.  The program
    arx.c expects 3 files, X.dat, P.dat, and Po.dat to exist in the
    directory it is called from.

    Let x = first column of X.dat
    Let S = second column of X.dat
    Output file T.dat file contains

    T(t) = a(0) + a(1)x(t-1) + a(2)x(t-2) + ... + a(Tx)x(t-Tx) +
           b(-Tac+1)S(t+Tac-1) + b(0)S(t-0) + ... +
	   b(1)S(t-1) + ... + b(Tc)S(t-Tc),

    where the coefficient vectors a and b are found in P.dat, and

    if t <= Tx,    T(t) = FLAG,
    if t >  K-Tac, T(t) = FLAG, and
    if t <= Tc,    T(t) = FLAG.

    X.dat contains the input data.  Each of its K rows is an
    observation and each column is a variable.  The first column is
    the autoregressive data.  The second column is the exogenous data.
    Currently, only one exogenous input variable is allowed.

    T.dat is the arx.o output.  It will have one column and K rows,
    each row corresponding to the input in the repective row in X.dat.
    If insufficient data existed in X.dat to make the full
    calculation, that row is set to FLAG in T.dat. If 0.99 times any
    element used in sum to compute T(t) is greater than FLAG, T(t) is
    set to FLAG.

    Po.dat contains auxiliary information.  Format of Po.dat file:

    T.dat   # The name of the output file to write
    X.dat   # The name of the data to read in
    P.dat   # The model parameters
    0       # Tac, the number of acausal terms
    3       # Tc, the number of causal terms
    3       # T, the number of AR terms
    1       # Nin, the number of exogenous inputs 
    1       # Nx, the number of spatial inputs 
    99999   # FLAG, the flag value to use (must be integer)
    0       # The level of verbosity

    P.dat contains the ARX coefficients.  File should have one column with rows

    a_(0)
    a_(1)
    a_(2)
    a_(3)
    ...
    a_(T)
    b_(-Tac+1)
    b_(-Tac+2)
    ...
    b_(-Tac+Tac)
    b_(1) 
    b_(2) 
    ...
    b_(Tc)

    If P.dat has Nx columns (with the same number of rows) output file T.dat
    will have Nx columns, with each column corresponding to operation done using
    parameters in respective column in P.dat.

    Version 0.3 ---- R.S. Weigel March 20, 2003.
    To do: Use malloc. 
    To do: Test memory limits and change doubles to 8 digits?

 */

#include <stdio.h>
#include <math.h>
#include <string.h>

/* Change this to use malloc */
#define NMAX_rows 100000
#define NMAX_spatial 100
#define NMAX_inputs 3

FILE    *Po_ptr,*P_ptr,*X_ptr,*T_ptr,*L_ptr;

int    Po_load(char Po[]);
int    P_load();
int    X_load();
int    Run_model();
int    T_write();

char    file_P[80];
char    file_T[80];
char    file_X[80];
char    file_Po[80];
char    file_default_Po[] = "Po.dat";

int     N           = 0;
int     Tc          = 0;
int     Tac         = 0;
int     Tx          = 0;
int     Nx          = 0;
int     Nin         = 0;
int     M           = 0;
int     K           = 0;
int     Verbosity   = 0;
int     stat        = 0;
int     nofail      = 1;
int     badpt       = 0;
int     Ts          = 0;

double  FLAG        = 99999;
double  P[NMAX_rows][NMAX_spatial];
double  p[NMAX_rows];
double  T[NMAX_rows][NMAX_inputs][NMAX_spatial];
double  TT[NMAX_rows][NMAX_spatial];
double  X[NMAX_rows];
double  XX[NMAX_rows][NMAX_inputs];

main (int argc, char *argv[]) {  

    if (Verbosity > 0){
      printf("------------------------------------------\n");
      printf("Entering function main of arx.c.\n");
      printf("------------------------------------------\n");
    }

  if (argc == 1){
    if (0) {
    printf("------------------------------------------\n");
    printf("No initialization file specified on command line.\n");
    printf("Will attempt to using default name of Po.dat.\n");
    printf("------------------------------------------\n");
    }
    strcpy(file_Po,file_default_Po);
  }
  else {
    strcpy(file_Po,argv[1]);
  }

  nofail = Po_load(file_Po); if (nofail == 0) return; /* Read in Po file   */
  nofail = P_load();         if (nofail == 0) return; /* Read in P file    */
  nofail = X_load();         if (nofail == 0) return; /* Read in X file    */
  nofail = Run_model();      if (nofail == 0) return; /* Run model         */

    if (Verbosity > 0){
      printf("------------------------------------------\n");
      printf("Exiting function main of arx.c.\n");
      printf("------------------------------------------\n");
    }


}

/************************************************************/
int Run_model(){

  int k        = 0;
  int j        = 0;
  int i        = 0;
  int z        = 0;
  int set_flag = 0;
  double temp  = 0.0;
  double sumj  = 0.0;
  double sumi  = 0.0;

  if (Verbosity > 0) {
    printf("------------------------------------------\n");
    printf("Start running the model\n");
    printf("------------------------------------------\n");
  }

  for (z = 0; z < Nx ; z++){
  if (Verbosity > 1) {
    printf("------------------------------------------\n");
    printf("AR part\n");
    printf("------------------------------------------\n");
  }
  /************************************************************/
  /* AR part */
  /************************************************************/
  for (k = 0 ; k < K ; k++){
    sumj       = 0.0;
    temp       = 0.0;
    T[k][0][z] = 0.0;
    badpt      = 0;

    if (k > Ts+Tx-1) {
      for (j = 1 ; j < (Tx+1) ; j++) {
	if (Verbosity > 1){
	  printf("k = %d P[%d][%d] = %lf XX[%d][0] = %lf\n",
		 k,j,z,P[j][z],k-j,XX[k-j-Ts][0]);
	}
	if (abs(XX[k-j-Ts][0]) > 0.99*FLAG){badpt = 1;}
	temp  = P[j][z]*XX[k-j-Ts][0];
	sumj  = sumj + temp;
      }

      T[k][0][z] = sumj + P[0][z];             /* Bias term */

      if (badpt == 1){
	T[k][0][z] = FLAG;
      }

    }

    if (Verbosity > 1){
      printf("T[%d][%d][%d] = %lf\n",k,0,z,T[k][0][z]);
    }


  }
  /************************************************************/
  if (Verbosity > 1) {
    printf("------------------------------------------\n");
    printf("Exogenous part\n");
    printf("------------------------------------------\n");
  }
  /************************************************************/
  /* Exogenous part */
  /************************************************************/
  for (i = 1 ; i < Nin + 1 ; i++){
  for (k = 0 ; k < K ; k++){
    sumj       = 0.0;
    temp       = 0.0;
    T[k][i][z] = 0.0;
    badpt      = 0;

    /* Causal Part of Exogenous */
    if (k > Ts+Tc-1) {
      for (j = 1 ; j < (Tc+1) ; j++) {
	if (Verbosity > 1){
	  printf("k = %d P[%d][%d] = %lf XX[%d][%d] = %lf\n",
		 k,Tx+Tac+j,z,P[Tx+Tac+j][z],k-j,i,XX[k-j-Ts][i]);
	}
	if (abs(XX[k-j-Ts][i]) > 0.99*FLAG){badpt = 1;}
	temp  = P[Tx+Tac+j][z]*XX[k-j-Ts][i];
	sumj  = sumj + temp;
      }
    }

    /* Acausal Part of Exogenous*/
    if (k < K-Tac+Ts) {
      for (j = 1 ; j < (Tac+1) ; j++) {
	if (Verbosity > 1){
	  printf("k = %d P[%d][%d] = %lf XX[%d][1] = %lf\n",
		 k,z,Tx+Tac-j+1,P[Tx+Tac-j+1][z],k+j,XX[k+j-1-Ts][i]);
	}
	if (abs(XX[k+j-1-Ts][i]) > 0.99*FLAG){badpt = 1;}
	temp  = P[Tx+Tac-j+1][z]*XX[k+j-1-Ts][i];
	sumj  = sumj + temp;
      }
    }

    T[k][i][z] = sumj;

    if (badpt == 1){
      T[k][i][z] = FLAG;
    }
    if (Verbosity > 1){
      printf("T[%d][%d][%d] = %lf\n",k,i,z,T[k][i][z]);
      //printf("\n");
    }
  }
  }
  /************************************************************/

  /************************************************************/
  /* Sum over columns (put this in above loop to speed up)
  /************************************************************/
  for (k = 0 ; k < K ; k++){
    sumi  = 0.0;
    badpt = 0;
    for (i = 0 ; i < Nin + 1; i++) {
      TT[k][z] = T[k][i][z] + sumi;
      sumi     = TT[k][z];
      if (abs(T[k][i][z]) > 0.99*FLAG){badpt = 1;}
    }
    if (badpt == 1){
      TT[k][z] = FLAG;
    }
    if (1){
    if (k <= Ts+Tx-1)  {TT[k][z] = FLAG;}
    if (k <= Ts+Tc-1)  {TT[k][z] = FLAG;}
    if (k >= K-Tac+Ts) {TT[k][z] = FLAG;}
    }
  }

  } 

  if (Verbosity) {
    printf("------------------------------------------\n");
    printf("Done running the model\n");
    printf("------------------------------------------\n");
  }

  nofail = T_write();        

  if (nofail == 0) return 0;        /* Write output file */

  return 1;

}
/************************************************************/
int Po_load(char file_Po[]){

  int     fileend     = 1;
  char    commentstr[100];

  if (Verbosity) {
    printf("------------------------------------------\n");
    printf("Data from initiation file %s\n",file_Po);
    printf("------------------------------------------\n");
  }

  if ((Po_ptr = fopen(file_Po,"r")) == NULL){
    printf("ERROR: Cannot open initiation file: %s. Aborting\n",file_Po);
    return 0;
  }

  fileend = fscanf(Po_ptr,"%s %[^\n]",&file_T,&commentstr);
  fileend = fscanf(Po_ptr,"%s %[^\n]",&file_X,&commentstr);
  fileend = fscanf(Po_ptr,"%s %[^\n]",&file_P,&commentstr);
  fileend = fscanf(Po_ptr,"%d %[^\n]",&Tac,&commentstr);
  fileend = fscanf(Po_ptr,"%d %[^\n]",&Tc,&commentstr);
  fileend = fscanf(Po_ptr,"%d %[^\n]",&Tx,&commentstr);
  fileend = fscanf(Po_ptr,"%d %[^\n]",&Ts,&commentstr);
  fileend = fscanf(Po_ptr,"%d %[^\n]",&Nin,&commentstr);
  fileend = fscanf(Po_ptr,"%d %[^\n]",&Nx,&commentstr);
  fileend = fscanf(Po_ptr,"%lf %[^\n]",&FLAG,&commentstr);
  fileend = fscanf(Po_ptr,"%d %[^\n]",&Verbosity,&commentstr);

  fclose(Po_ptr);

  if (Verbosity) {
    printf("------------------------------------------\n");
    printf("Data from initiation file: %s\n",file_Po);      
    printf("------------------------------------------\n");
    printf("T file       %s\n",file_T);    
    printf("X file       %s\n",file_X);    
    printf("P file       %s\n",file_P);    
    printf("Tac          %d\n",Tac);    
    printf("Tc           %d\n",Tc);    
    printf("Tx           %d\n",Tx);    
    printf("Ts           %d\n",Ts);    
    printf("Nin          %d\n",Nin);    
    printf("Nx           %d\n",Nx);    
    printf("FLAG         %lf\n",FLAG);    
    printf("Verbosity    %d\n",Verbosity);    
  }

  if (Nin > 1){
    printf("ERROR: Nin must be 0 or 1.\n");
    printf("ERROR: Multiple inputs not currently supported. Aborting\n");
    return 0;
  }

  return 1;
}
/************************************************************/
int P_load(){

  int     fileend     = 1;
  int     i           = 0;
  int     k           = 0;
  int     z           = 0;
  float   temp        = 0;

  if (Verbosity) {
    printf("------------------------------------------\n");
    printf("Data from model parameter file: %s\n",file_P);      
    printf("------------------------------------------\n");
  }

  if ((P_ptr = fopen(file_P,"r")) == NULL){
    printf("ERROR: Cannot open parameter file named %s. Aborting.\n",file_P);
    return 0;
  }

  if (Tx == 0) {
    if (Verbosity > 0) {
      printf("Found T = 0.  Model is effectively a FIR filter\n");
    }
  }
  if (Tc + Tac == 0) {
    if (Verbosity > 0) {
      printf("Found Ta + Tac = 0.  Model is effectively an AR filter\n");
    }
  }

  while (fileend != EOF){

    /* Reads file from left to right, then top to bottom*/
    fileend = fscanf(P_ptr,"%lf",&p[i]);

    if (fileend != EOF){    

      if (z == Nx) {
	z = 0;
	k = k + 1;
      }

      P[k][z] = p[i];

      if (Verbosity > 1) {
	printf("P[k=%d][z=%d] = %lf\n",k,z,P[k][z]);
      }
      z = z + 1;
    }
  }  
  
  if (k >= NMAX_rows){
    printf("ERROR: Number of rows in %s exceeds maximum");
    printf("allowed (%d). Aborting.\n",file_P,NMAX_rows);
    return 0;
  }
  
  if (z >= NMAX_spatial){
    printf("ERROR: Number of columns in %s exceeds maximum");
    printf("allowed (%d). Aborting.\n",file_P,NMAX_spatial);
    return 0;
  }
  
  if (k+1 !=  Tc + Tac + Tx + 1) {
    printf("ERROR: Incorrect number of parameters in P.dat (%d). ",k+1);
    printf("Expected %d. Aborting.\n",Tc+Tac+Tx+1);
    return 0; 
  }

  fclose(P_ptr);
  return 1;

}
/************************************************************/
int X_load(){

  int     fileend     = 1;
  int     i           = 0;
  int     k           = 0;
  int     z           = 0;
  float   temp        = 0;

  if ((X_ptr = fopen(file_X,"r")) == NULL){
    printf("ERROR: Cannot open input data file %s. Aborting.\n",file_X);
    return 0;
  }

  if (Verbosity) {
    printf("------------------------------------------\n");
    printf("Loading input data from file %s\n",file_X);      
  }

  while (fileend != EOF){

    /* Reads file from left to right, then top to bottom*/
    fileend = fscanf(X_ptr,"%lf",&X[i]);

    if (fileend != EOF){    

      if (z == Nin + 1) {
	z = 0;
	k = k + 1;
      }

      XX[k][z] = X[i];

      if (Verbosity > 1) {
	printf("XX[k=%d][z=%d] = %lf\n",k,z,XX[k][z]);
      }

      z = z + 1;

    }
  
    if (k >= NMAX_rows){
      printf("ERROR: Number of rows in %s exceeds maximum");
      printf("allowed (%d). Aborting.\n",file_X,NMAX_rows);
      return 0;
    }
  
    if (z >= NMAX_inputs){
      printf("ERROR: Number of columns in %s exceeds maximum");
      printf("allowed (%d). Aborting.\n",file_X,NMAX_inputs);
      return 0;
    }

  }

  K = k+1;

  if (Verbosity) {
    printf("Number of rows in input file = %d\n",K);
    printf("------------------------------------------\n");

  }

  fclose(X_ptr);
  
  return 1;

}

/************************************************************/
int T_write(){

  int     fileend     = 1;
  int     j           = 0;
  int     z           = 0;

  if ((T_ptr = fopen(file_T,"w")) == NULL){
    printf("ERROR: Cannot open output data file %s. Aborting.\n",file_T);
    return 0;
  }

  if (Verbosity) {
      printf("------------------------------------------\n");
      printf("Output data written to %s\n",file_T);      
    }

  for (j = 0; j < K; j++) {
    for (z = 0; z < Nx; z++) {
      fprintf(T_ptr,"%lf ",TT[j][z]);
      if (Verbosity > 1) {
	printf("TT[%d][%d] = %lf ",j,z,TT[j][z]);
      }
    }
    if (Verbosity > 1) {
      printf("\n",j,z,TT[j][z]);
    }
    fprintf(T_ptr,"\n",TT[j][z]);
  }

  fclose(T_ptr);

  if (Verbosity) {
      printf("Number of rows in output file = %d\n",K);      
      printf("------------------------------------------\n");
    }

  return 1;
}
/************************************************************/






