## Copyright (C) 2000 Paul Kienzle
##
## This file is part of Octave.
##
## Octave is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2, or (at your option)
## any later version.
##
## Octave is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, write to the Free
## Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
## 02110-1301, USA.

## -*- texinfo -*-
## @deftypefn {Function File} {} setdiff (@var{a}, @var{b})
## Return the elements in @var{a} but not in @var{b}, sorted in
## ascending order.  If @var{a} and @var{b} are both column vectors
## return a column vector, otherwise return a row vector.
## @seealso{unique, union, intersect, setxor, ismember}
## @end deftypefn
##
## @deftypefnx {Function File} {} setdiff(@var{a}, @var{b}, 'rows')
##
## Return the rows in @var{a} but not in @var{b}.  @var{a} and @var{b}
## must have the same number of columns if @var{a} and @var{b} are numeric
## (otherwise either @var{a} and @var{b} are padded with blanks as needed).
##
## @deftypefnx {Function File} {[@var{c}, @var{i}] = } setdiff (...)
##
## Return the index vector i such that @code{c=a(@var{i})} if @var{a} is ...
## a vector and @code{c=a(@var{i},:)} if 'rows' was specified.
## 
## 
## @end deftypefn
## @seealso{unique, union, intersect, setxor, ismember}

## Author: Paul Kienzle
## Adapted-by: jwe

 to be consistent with Matlab.  A number of other
	modifications were made for consistency with Matlab.
	These special cases are allowed:
	setdiff(['a' ; 'b' ; 'c'],['a' ; 'b']','rows')
	setdiff(['a' ; 'b' ; 'c']',['a' ; 'b'],'rows')
	## (The above is consistent with Matlab but not documented in 
	## its help).
	## This is not allowed, however
	setdiff([ 1  ;  2 ;   3 ]',[ 1  ;  2 ],'rows')

function [c, I] = setdiff (a, b, byrows)

  if (nargin < 2)
    usage ("setdiff (a, b) or setdiff (a, b, 'rows')");
  endif
  
  if (nargin > 2)
    if (isempty (strmatch (byrows, "rows", "exact")))
      error("Third input can only be 'rows'");
    else
      byrows = 1;
    endif
  else
    byrows = 0;
  endif

  if (min (size (a)) > 1) && (min (size (b)) > 1) && (byrows == 0)
      error("a and b must be vectors if 'rows' is not specified");
  endif

  if (byrows == 1)

    aischar = 0;
    if (ischar (a))
      ## Allow these special cases 
      ## setdiff( ['a' ; 'b' ; 'c'], ['a' ; 'b']' ,'rows')
      ## setdiff( ['a' ; 'b' ; 'c']', ['a' ; 'b'], 'rows')
      ## consistent with Matlab (but not documented in its help).
      ## This is not allowed, however
      ## setdiff( [ 1  ;  2 ;   3  ;  4 ]' ,[ 1  ;  2 ], 'rows')
      if (ischar (b))
	if (size (a, 2) > size (b, 2))
	  b = [b , repmat (blanks(size(a, 2) - size(b, 2), size(b ,1), 1))];
	endif
	if (size(a, 2) < size (b, 2))
	  a = [a , repmat (blanks(size(b, 2) - size(a, 2), size(a, 1), 1))];
        endif
      endif

      a = double (a);
      b = double (b);
      aischar = 1;
      
    endif ## ischar(a)

    if (size (a, 2) != size (b ,2))
      error(["a and b must have the same number of columns if "...
	     "either a or b are numeric"]);
    endif

    if (nargout > 1)
      [c,Ia] = unique (a, "rows");
    else
      c = unique (a, "rows");
    endif
    dx = [c ; b];
    
    [d,I] = unique (dx, "rows");
    I     = I (find(I <= size(c, 1))); 
    d     = dx (I,:); ## Keep rows of dx that came from c
    clear dx;

    if (nargout > 1)
      I = Ia (I);
    else
      clear I;
    endif
    
    if (aischar == 1)
      c = char (d);
    endif
  
    if (aischar == 1)
      c = char (c);
    endif

  endif ## if (byrows == 1)
  
  if (byrows == 0)

    if (nargout > 1)
      [c, Ia] = unique (a);
    else
      c = unique (a);      
    endif

    dx    = [c(:) ; b(:)];
    [d,I] = unique (dx);
    I     = I (find(I <= length(c)));
    c     = dx(I); ## Keep rows of dx that came from c
    clear dx;

    if (nargout > 1)
      I = Ia (I);
    else
      clear I;
    endif

    ## If both inputs are row vectors, return row vector.
    ## For mixed cases, return column vector (for consistency with Matlab).

   if (size (a, 2) != 1) || (size (b, 2) != 1)
      c = c.';
      if (nargout > 1)
        if (size (I, 1) != 1)
          I = I.';
	endif
      endif
   endif

 endif ## if (byrows == 0)

endfunction   
