## Copyright (C) 2000  Etienne Grossmann
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

## -*- texinfo -*-
## @deftypefn {Function File} {} xrmpath (@var{dir1}, @dots{})
## Remove @var{dir1}, @dots{} from the current @code{LOADPATH}.
## A warning is generated if any of the paths are not in LOADPATH.
##
## Compound paths are allowed
## @example
## xrmpath ("/path1:/path2","path3")
## @result{} Removes path1, path2, and path3 from LOADPATH.  
## @end example
##
## Used as above, this function equivalent to Matlab's rmpath except 
## that "//" syntax (see LOADPATH help) is allowed.
## Use of "//" syntax will result in a warning if warn_matlab_incompatable 
## = 1.
## 
## P = xrmpath(DIR1,...) returns LOADPATH prior to modification.
##
## @seealso{LOADPATH, addpath, savepath, setpath}
## @end deftypefn

## Author: Etienne Grossmann <etienne@cs.uky.edu>

function ret = xrmpath (varargin)

  if (nargout > 0)
    ret = LOADPATH;
  end

  # If last argument is zero, do not warn if directory does not valid.
  warn_not_in_path = 1;
  if (length(varargin)>0)
    if (varargin{end} == 0) 
      varargin = varargin(1:end-1);
      warn_not_in_path = 0;
    end
  end  

  tmp = cleanpath([LOADPATH,DEFAULT_LOADPATH]);
  LOADPATH = ['.',tmp(1:end-1)];
  DEFAULT_LOADPATH = ''; ## (Setting to '' generates warning)

  if (nargout > 0)
    ret = LOADPATH;
  end

  varargin = cleanpath(varargin); ## One (cleaned) path per cell

  for arg = 1:length (varargin)

    p = varargin{arg};
    lo = length (path);

    tmp = strrep ([LOADPATH,':'], sprintf("%s", p), ":");
    LOADPATH = tmp(1:end-1);

    if (lo == length (LOADPATH)) && (warn_not_in_path)
      warning('xrmpath: Directory "%s" is not in path',p(2:end-1))
    end

  end
  
endfunction  

%!assert(rmpath(':',''),'');
%!assert(rmpath('hello:',''),'hello');
%!assert(rmpath('hello:world',''),'hello:world');
%!assert(rmpath(':hello:world',''),'hello:world');
%!assert(rmpath(':hello:world:',''),'hello:world');
%!assert(rmpath(':hello::world:',''),'hello:world');

%!assert(rmpath('hello','hello'),'');
%!assert(rmpath(':hello','hello'),':');
%!assert(rmpath('hello:','hello'),':');
%!assert(rmpath('hello:hello','hello'),'');
%!assert(rmpath('hello:hello:hello','hello'),'');
%!assert(rmpath('hello:hello:hello:hello','hello'),'');
%!assert(rmpath(':hello:hello','hello'),':');
%!assert(rmpath('hello:hello:','hello'),':');
%!assert(rmpath('hello','world'),'hello');
%!assert(rmpath(':hello','','hello'),'');
%!assert(rmpath(':hello','hello',''),'');

%!assert(rmpath('hello:world','hello','world'),'');
%!assert(rmpath('hello:world:','hello','world'),':');
%!assert(rmpath(':hello:world:','hello','world'),':');

%!assert(rmpath('hello:world','','hello','world'),'');
%!assert(rmpath('hello:world:','','hello','world'),'');
%!assert(rmpath(':hello:world:','','hello','world'),'');

%!assert(rmpath('hello:world','hello'),'world');
%!assert(rmpath('hello:world','world'),'hello');
%!assert(rmpath('hello:world:','hello'),'world:');
%!assert(rmpath('hello:world:','world'),'hello:');
%!assert(rmpath(':hello:world:','hello'),':world:');
%!assert(rmpath(':hello:world:','world'),':hello:');
