function ARV = arv_nonflag(y,f,FLAG,EXACT)
%ARV_NONFLAG Average relative variance between non-FLAG elements of actual and predicted vector.
% 
%   ARV(A,P) returns (1/N)*sum( (A-P).^2 )./(std(A,1))^2 assuming FLAG = NaN.
%
%   ARV is the mean square error scaled to the variance of the
%   actual vector, A.  It can be interpreted as the fraction of the
%   variance of A that is not predicted by P.
%
%   For matrices, ARV(A,P) returns a column vector with elements containing
%   the ARV between the respective columns of A and P.
%
%   ARV_NONFLAG(A,P,FLAG,EXACT) If EXACT = 0, returns arv of elements less
%   than FLAG.  EXACT = 1, returns arv of elements not equal to flag.  EXACT
%   = 2, returns arv of elements with absolute value less than FLAG.
%
%   If all elements along dimension DIM are equal to FLAG, a fill of FLAG is
%   used.
%   
%   See also ARV, MSE, *_NONFLAG.

% R.S. Weigel, 04/02/2004.
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
if (nargin < 3)
  FLAG     = NaN;
end
if (nargin < 4)
  EXACT = 1;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

if (prod(size(y)) ~= prod(size(f)))
  error('Inputs must be the same size');
end
  
if (size(y,1) == 1)
  y = y';
  f = f';
  if (length(y) < 2)
    error('Inputs must have 2 or more elements');
  end
end

for i = 1:size(y,2)
  Iy = []; If = [];
  if (EXACT == 0)
    Iy = find(y(:,i) < FLAG);
    If = find(f(:,i) < FLAG);
  end
  if (EXACT == 1)
    Iy = find(y(:,i) ~= FLAG);
    If = find(f(:,i) ~= FLAG);
  end
  if (EXACT == 2)
    Iy = find(abs(y(:,i)) < FLAG);
    If = find(abs(f(:,i)) < FLAG);
  end
  if (isnan(FLAG))
    Iy = find(isnan(y(:,i)) == 0);
    If = find(isnan(f(:,i)) == 0);
  end
  
  I  = intersect(Iy,If);
  if (length(I) > 1)
    meany    = repmat(mean(y(I,i),1),length(I),1);
    N        = sum((y(I,i) - f(I,i)).^2,1);
    D        = sum((y(I,i) - meany).^2,1);
    ARV(1,i) = N./D;
  else
    ARV(1,i) = FLAG;      
  end
end



