function V_out = sptransform(inSys,outSys,UT,V_in,debug)
%SPTRANSFORM Transform vectors between/within heliocentric and geocentric
%
%   V_out = SPTRANSFORM(inSys,outSys,UT,V_in) where inSys and outSys are
%   either integers or strings for any of the avalable coordinate systems
%   listed below.  If UT=[], UT = [2000 1 1 0 0 0] is used.
%
%   Each row in Nx3 matrix V_in is transformed at the given UT = [Year,
%   Month, Day, Hour, Minute, Second].  If UT has M rows, then V_in must be
%   a 1x3 vector.  V_out is a Mx3 matrix of V_in transformed at each
%   universal time given by the rows of matrix UT.
%   
%   If V_in and UT are Nx3 and Mx3 matrices, respectively, then V_out is
%   (M*N)x3 with elements of [ V_in(UT(1)) ; ... ; V_in(UT(end))].
%   
%   Available coordinate systems (inSys or outSys can be integer or strings):
%
%   0  or 'GEI'     3  or 'MAG'   6  or 'SM'    9  or 'HAE'  
%   1  or 'J2000'   4  or 'GSE'   7  or 'GSEQ'  10 or 'HEEQ' 
%   2  or 'GEO'     5  or 'GSM'   8  or 'HEE'  
%
%   The only allowed transform between Heliocentric and Geocentric systems
%   is HEE<->GSE.
%  

% R.S. Weigel 06/21/2005

% cxform.c seems to abort without explanation if Year is out of range.
% Catch the error here.
if (max(UT(:,1))>2008)
  warning(['Year > 2008 given.  Transform may not be known and '...
	  'execution may abort.']);
end
  
if isempty(UT)
  UT = [2000 1 1 0 0 0];
end

if (size(UT,2)<4) 
  Np = 3-size(UT,2);
  warning(['UT matrix has less than 3 columns. '...
	   'Missing Month and/or Day columns are '...
	   'assumed to be 1']);
  UT = [UT,zeros(size(UT,1),Np)];
end
if (size(UT,2)<6)
  warning(['UT matrix has less than 6 columns. Missing H,MIN,S columns '...
	   'are assumed to be zero.']);
end

Systems = ...
    {'GEI','J2000','GEO','MAG','GSE','GSM','SM','GSEQ','HEE','HAE','HEEQ'};  
  
if (size(V_in,1) > 1) & (size(UT,1) > 1)
%  error('If UT (V_in) is a matrix then V_in (UT) must be a row vector.');
end

if (isstr(inSys))
  Iin = strmatch(inSys,Systems,'exact');
  Iin = Iin-1;
  if isempty(Iin)
    error('Coordinate system %s does not exist',inSys);
  end
else
  Iin = inSys;
end

if (isstr(outSys))
  Iout = strmatch(outSys,Systems,'exact');
  Iout = Iout-1;
  if isempty(Iout)
    error('Coordinate system %s does not exist',outSys);
  end
else
  Iout = outSys;
end  

if  ( (Iin < 9) & (Iout > 8) ) | ( (Iout < 8) & (Iin > 7) )
  if  ( (Iin ~= 9) & (Iout ~= 5) ) | ( (Iout ~= 9) & (Iin ~= 5) )
    error(['The only allowed transform between Heliocentric and'...
	   ' Geocentric systems  is HEE<->GSE.']);
end
end

if (nargin < 5)
  debug = 0;
end

V_out = sptransform_oct(Iin,Iout,UT,V_in,debug);
