function [Attr,Z] = ts_get(in_name,Index)
%TS_GET Retrieve time series from data base.
%  
%    [Attr,Z] = TS_GET(Name)
%  
%    Name is a dataset name from the list of available time series data
%    sets.  Use TS_LIST for a list of possibilities.
%
%    Data will be returned in Z if NAME matches the first length(Name)
%    characters of any dataset name.  If there are multiple matches, data
%    for first match is returned.
%
%    Example: Return data for the first time series returned by TS_LIST.
%    (Copy this to the command line).
%    Names_c = ts_list; [Attr,Z] = ts_get(Names_c{1}); Attr, Z
%    
%    See also TS_GET_DEMO, TS_LIST, TS_SLICE.

% R.S. Weigel 11/11/2004

TSDS_GLOBALS;

if (nargin < 3)
  Attr = ts_list(in_name);
end

if (isempty(Attr)) | iscell(Attr)
  error('Name not found or name not unique');
else
  fname = Attr.Filename;
end

if (is_unix == 1)
  J  = findstr(fname,'/');
end
if (is_unix == 0)
  slash = '\\'; % Octave does not like a lone forward slash.
  J     = findstr(fname,slash(1));
end

fname_full   = fname;
fname        = fname_full(J(end)+1:end);
fname_path   = fname_full(1:J(end));
fname_path   = TSDS_DATA_LOCAL_WRITE;

fprintf80(sprintf('ts_get        : Reading directory %s\n',fname_path));

D = readdir(fname_path);

% Octave's readdir function changed sometime after 2.1.50 (output is cell).
% instead of char array.  This will break on Octave versions after 2.1.50
% that output a cell and if strvcat function cannot handle cell arrays.

if (iscell(D))
  if (is_octave)
    Dm = [];
    for i = 1:length(D)
      Dm = strvcat(Dm,D{i});
    end
    D = Dm;
  else
    D = strvcat(D);
  end
end

islocal = 0;

if (nargout > 1)

% Avoid slow for loop
D_FLAT = D';
D_FLAT = D_FLAT(:)';

I = findstr(D_FLAT,fname);
if (~isempty(I))
    islocal = 1;
    fprintf80(sprintf('ts_get        : Data is in local file %s\n',fname));
end

if (0)
%for i = 1:size(D,1)
%  % Is the file is in the non-writable TSDS_DATA directory
%  if (~isempty(strmatch(fname,D(i,:),'exact')))
%    islocal = 1;
%  end
%end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% If data file is not found, attempt download
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
success  = 0;

if (islocal == 0)

  % If not, try to get it.
  fprintf80(...
      sprintf('ts_get        : Requested data is in file \n%s\n',fname));
  fprintf80(...
      sprintf('ts_get        : Did not find file in dir %s\n',fname_path));
  fprintf80(...
      sprintf('ts_get        : Attempting to download this file.\n'));

  success = ts_download(fname);
  
  if (success == 0)
    fprintf80('ts_get        : Aborting because download failed.');
    error('ts_get.m')
  end

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% If data set found and data was requested
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (~isempty(Attr))

  fprintf80(...
      sprintf('ts_get        : Extracting (from file) data set: %s',in_name));
  tic;

  % Get the data
  if (nargin < 2)
    Z = double(ts_dfread(Attr)); 
  else
    Z = double(ts_dfread(Attr,Index));
  end
  
%  fprintf80(sprintf('ts_get        : Time to read data from file: %.3f sec',toc));

  % Print warning info to screen.
  if isfield(Attr,'Warning');
    Warning = Attr.Warning;
    if (length(deblank(Warning))>1)
      fprintf('\n');
      str=sprintf(['ts_get        : '...
		   'Warning for data set "%s": %s'],in_name,Warning);
      fprintf80(str);
      fprintf('\n');
    end
  end

  % Print acknowledgement info to screen
  if (TSDS_ACK_WARN == 1) 

    if isfield(Attr,'Acknowledgement')
      Ack = Attr.Acknowledgement;
    end
    if isfield(Attr,'Acknowledge') % For broken metadata files 
      Ack = Attr.Acknowledge;
    end

    if (length(deblank(Ack))>1)
      str=sprintf(['ts_get        : '...
		   'Acknowledgement information for data' ...
		   ' set "%s": %s'],in_name,Ack);
      fprintf('\n');
      fprintf80(str);
      fprintf('\n');
    end
  end
  
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end % if (nargout > 1)
