function ts_insert(Attr_in,Var_in,basename,clean,regen_db)
%TS_INSERT Insert a time series into local data base.
%
%   TS_INSERT(Attr_cell,Var_cell) Inserts the time series in Var_cell with
%   attributes in Attr_cell in to the local data base.  The elements of
%   Attr_cell and Var_cell can be cell arrays.
%  
%   TS_INSERT(Attr_cell,Var_cell,basename) Creates files with tag basename.
%
%   TS_INSERT(Attr_cell,Var,basename,clean) If clean = 'clean', removes all
%   files matching *basename* before processing.  basename can be [];
%
%   TS_INSERT(...,clean,regen_db) will not regenerate data base (read by
%   TS_LIST) if regen_db = 0.  If clean is [], no files are removed before
%   processing.  If regen_db=1 (default), entire data base is regenerated
%   based on *metadataonly* files found in the data directory.  If
%   regen_db=2, only new *metadata* files are read and added to data base
%   (not implemented).
% 
%   Var_cell can contain a cell array with elements of strings containing
%   the full path of a .mat file with a single variable named Var that is to
%   be associated with the corresponding element of Attr_cell (use this when
%   Var is large).
%
%   See also: TS_INSERT_DEMO.

TSDS_GLOBALS;

if nargin < 3
  basename = '';
end

if nargin < 4
  clean    = '';
end

if nargin < 5
  regen_db = 1;
end

k = 1;

% If Var_in is a cell array of filenames
if iscell(Var_in)
  if (isstr(Var_in{1}))

    regen_db = 0;

    i = 1;

    while i <= length(Var_in)

      if i > 1;
	clean = ''; % Don't allow clean if multiple files to be inserted
      end

      fprintf80(sprintf(['ts_insert    : Processing file %d of %d'],...
			i,length(Var_in)));

      Nb = 0;
      k  = 1;

      while (Nb < TSDS_MAX_FILESIZE) & (i <= length(Var_in))

	FILE     = load(Var_in{i});

	xVar{k}  = FILE.Var;
	xAttr{k} = Attr_in{i};

	if (is_octave == 0)
	  W      = whos('-file',Var_in{i});
	  tempNb = cat(1,W.bytes); % Sometimes W.bytes returns two
                                   % arrays.  Why?
	  tempNb = tempNb(end);    % In the case where it does return two
                                   % arrays the second one is correct.
	else
	  W      = stat(Var_in{i});
	  tempNb = W.size;
	end

	Nb = Nb + sum(tempNb(:));

	i = i+1;
	k = k+1;
      
      end

      if (k > 1)
	fprintf80(...
	    sprintf('ts_insert    : Combining %d files (%.1f Mb)',...
		    k-1,(Nb/1e6)));
      end

      if (i == length(Var_in)+1)
	regen_db = 1;
      end

      ts_insert(xAttr,xVar,basename,clean,regen_db);
      
      clear xAttr xVar;
      
    end

    return;
  
  end

end

% Allow input to be a the basic attribute structure
if ~iscell(Attr_in)
  Attr_cell{1} = Attr_in;
  Var_cell{1}  = Var_in;
  k = 2;
end

% Allow input to be a cell array with elements of cell arrays  
subcells = 0;
if iscell(Attr_in)
  for j = 1:length(Attr_in)
    if (iscell(Attr_in{j}))    
      subcells = 1;
      break;
    end
  end
end

if (subcells == 1)
  for j = 1:length(Attr_in)
    if (iscell(Attr_in{j}))
      for i = 1:length(Attr_in{j})
	Attr_cell{k} = Attr_in{j}{i};
	Var_cell{k}  = Var_in{j}{i};
	k = k+1;
      end
    else
      Attr_cell{k} = Attr_in{j};
      Var_cell{k}  = Var_in{j};
      k = k+1;
    end
  end
end

if (k > 1)
  for i=1:length(Attr_cell)
    tmp{i} = Attr_cell{i}.Name;
  end
else
  for i=1:length(Attr_in)
    tmp{i} = Attr_in{i}.Name;
  end
end

strs = strvcat(tmp{:});
I = repeats(strs,'rows');

if (~isempty(I))
  strs(I,:)
  error(...
      'ts_insert      : One or more repeated names in input attribute list');
end

tmp = ts_db;
strs = strvcat(tmp.SET_NAMES,strs);
I = repeats(strs,'rows');

if (~isempty(I))
  strs(I,:)
  if (length(I)>1)
    error(...
	['ts_insert      : Above names in input list are already used.' ...
	 ' Data set names must be unique.']);
  else
    error(...
	['ts_insert      : Above name in input list is already used.' ...
	 ' Data set names must be unique.']);
  end    
end


if (k > 1) % If subcells

  ts_check(Attr_cell,Var_cell,basename);
  ts_mat(Attr_cell,Var_cell,basename,clean,regen_db);
%  ts_hdf(Attr_cell,Var_cell,basename,clean,regen_db);

else

  ts_check(Attr_in,Var_in,basename);
  ts_mat(Attr_in,Var_in,basename,clean,regen_db);  
%  ts_hdf(Attr_in,Var_in,basename,clean,regen_db);

end

ts_db('user',1); % Regenerate user data base.
ts_db('combined',1); % Regenerate combined data base.
