function NAMES = ts_list(in_name,pattern_opt,db_opt)
%TS_LIST Retrieves a time series from TSDS data base.
%  
%   TS_LIST prints to screen a list of names of available time series and
%   the file they are located in.
%  
%   Names_c = TS_LIST returns the list of all data set names in cell array
%   Names_c.
%
%   TS_LIST(Name) prints to screen information associated with the time
%   series having a name with its first N characters matching the string
%   Name, where N is the number of characters in Name (so that the full
%   name does not have to be input).
%
%   Attr_c = TS_LIST(Name) returns attributes in a cell array Attr_c.  This
%   is equivalent to the command Attr_c = TS_GET(Name)
%  
%   If there are multiple time series name matches, the first match is used.
%   If no matches, then list of time series names that have the string Name
%   in it is listed to the screen.
%
%   TS_LIST(Name,'include') or TS_LIST(Name,'exclude') lists time series
%   names that contain/do not contain the string Name.  For example, if you
%   are not interested in ground magnetometer data, use
%   ts_list('DMI','exclude') to shorten the list by removing any ground
%   magnetometer time series from the DMI World Data Center.
%
%   TS_LIST('user') or TS_LIST('base') lists only time series names in user
%   or base directory.
%
%   Example: 
%   Print information on first time series. (Copy this to the command line).
%   Names_c = ts_list; ts_list(Names_c{1})
%   
%   See also TS_GET, TS_DB.

% R.S. Weigel 11/11/2004

TSDS_GLOBALS;

if (nargin<1)
  in_name = '';
end
if (nargin<2)
  pattern_opt = 'include';
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Print attributes associated with in_name to screen
underscores80 = repmat('_',1,80);
if ~isempty(in_name) & (nargout == 0) & ...
      isempty(strmatch(in_name,'base','exact')) & ...
      isempty(strmatch(in_name,'user','exact')) & ...
      isempty(strmatch(in_name,'combined','exact'))
  fprintf('\n');
  fprintf('%s\n',underscores80);
  Attr = ts_list(in_name,pattern_opt);
  if (length(Attr)==1)
    Attr
  end
  fprintf('%s\n',underscores80);
  fprintf('\n');
  NAMES = [];
  return;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (~isempty(in_name))
  if ~isempty(strmatch(in_name,'base','exact')) | ...
	~isempty(strmatch(in_name,'user','exact'))
    TS_DBs = ts_db(in_name);
  else
    TS_DBs = ts_db('combined');
  end
else
  TS_DBs = ts_db('combined');
end

SET_NAMES   = TS_DBs.SET_NAMES;
SET_YEAR_O  = TS_DBs.SET_YEAR_O;  
SET_YEAR_F  = TS_DBs.SET_YEAR_F; 
SET_NUM_INFILE = TS_DBs.SET_NUM_INFILE; 	  
SET_FILE_NUM = TS_DBs.SET_FILE_NUM;
SET_DIR_NUM = TS_DBs.SET_DIR_NUM;
FILENAMES   = TS_DBs.FILENAMES;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Print all files and ts names to screen
if  (nargout == 0)
  for i = 1:length(FILENAMES)

    if (SET_DIR_NUM(i) == '0')
      base_dir = TSDS_DATA_LOCAL;
    else
      base_dir = TSDS_DATA_LOCAL_WRITE;
    end
    
    fname = FILENAMES{i};
    a     = findstr('metadataonly',fname);
    fname = [fname(1:a(1)-2),fname(a(1)+12:end)]; % Remove -metadataonly string

    fprintf('%s\n',repmat('_',1,80));
    fprintf80(sprintf('ts_list       : File %s:',fname));
%    fprintf80(sprintf('ts_list       : File %s in ',fname));
%    fprintf80(sprintf('ts_list       : directory %s ',base_dir));
%    fprintf80(sprintf('ts_list       : contains: '));

    I = find(SET_NUM_INFILE==1); % Find first set number in a file

    a = I(i);
    if (i+1 <= length(I))
      b = I(i+1) - 1;
    else
      b = a+SET_NUM_INFILE(end)-1;
    end

    for j = a:b

      name  =  deblank(SET_NAMES(j,:));
      year_o = deblank(SET_YEAR_O(j,:));
      year_f = deblank(SET_YEAR_F(j,:));
      fprintf80(sprintf('              : %s [%s-%s]',name,year_o,year_f));
    
    end
  
  end

end  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Return list of time series names
if (isempty(in_name)) & (nargout == 1)
  for j = 1:size(SET_NAMES,1)
    NAMES{j,1} = deblank(SET_NAMES(j,:));
  end
end  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get information
if (nargin > 0) & (nargout > 0)

  % Ignore any text in name that starts after a square bracket.  
  Ibracket = findstr('[',in_name);
  if (~isempty(Ibracket))
    in_name = in_name(1:Ibracket-1);
  end

  fprintf80(...
      sprintf(...
	  ['ts_list       : Looking in metadata for time series with exact '...
	   'name: %s.'],in_name));

  JTEMP = strmatch(in_name,SET_NAMES,'exact');
  k     = length(JTEMP); % k is number of times 
			 % in_name = SET_NAMES(1:length(in_name))

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Exact matches			 
  if (k == 1)		       
    fprintf80(sprintf(['ts_list       : Found exact match:'],k));
    name  =  deblank(SET_NAMES(JTEMP,:));
    year_o = deblank(SET_YEAR_O(JTEMP,:));
    year_f = deblank(SET_YEAR_F(JTEMP,:));
    fprintf80(sprintf('              : %s [%s-%s]',name,year_o,year_f));
  end
  
  if (k > 1)
    fprintf80('ts_list       : Error: duplicates in name list.');
    fprintf80(...
	sprintf('ts_list       : Name: ''%s'' was found in',...
		deblank(SET_NAMES(JTEMP(1),:))));
    for i = 1:k
      if (SET_DIR_NUM(JTEMP(i)) == '0')
	base_dir = TSDS_DATA_LOCAL;
      else
	base_dir = TSDS_DATA_LOCAL_WRITE;
      end
      fprintf80(sprintf('ts_list       : dir %s',base_dir));
      fprintf80(sprintf('ts_list       : file %s',...
			FILENAMES{SET_FILE_NUM(JTEMP(i))}));
      if i < k
	fprintf80(sprintf('ts_list       : and'));
      end
    end
    fprintf80(['ts_list       : Deleting all but one of the above files'...
	      ' and executing ']);
    
    fprintf80(['ts_list       : ts_db(''user'',1); ts_db(''combined'',1) '...
	       'may fix this error. Aborting.']);
    error('ts_list: Duplicates in name list');
  end
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
  % Check for partial matches.
  k2 = [];
  if (k == 0)     
    z = 1;
    fprintf80(...
	sprintf(['ts_list       : Exact name'...
		   ' not found.  Looking for partial matches.'],...
		in_name));
    
    % Avoid slow for loop
    SET_NAMES_FLAT = SET_NAMES';
    SET_NAMES_FLAT = SET_NAMES_FLAT(:);

    if (is_octave == 1)
      I = findstr(lower(SET_NAMES_FLAT),lower(in_name));
    end
    if (is_octave == 0)
      % Matlab's findstr does not like
      % blanks.
      I = regexp(lower(SET_NAMES_FLAT'),lower(in_name)); 
    end
    
    if (~isempty(I))
      k2 = 1+floor(I/size(SET_NAMES,2));
      k2 = unique(k2); % Remove repeated indices due to double match in name.
    end
    
    if (~isempty(strmatch('exclude',pattern_opt)))  
      if (~isempty(k2))
	k2 = setxor(k2,[1:size(SET_NAMES,1)]);
      end
    end
    
    if (~isempty(strmatch('include',pattern_opt)) & ~isempty(k2))
      fprintf80(...
	  sprintf(...
	      'ts_list       : Found %d name(s) with pattern /%s/',...
	      length(k2),in_name));
    end
    
    if (~isempty(strmatch('exclude',pattern_opt)) & ~isempty(k2))
      fprintf80(...
	  sprintf(...
	      'ts_list       : Found names without pattern /%s/',in_name));
    end

    for i = 1:length(k2)
      name  =  deblank(SET_NAMES(k2(i),:));
      year_o = deblank(SET_YEAR_O(k2(i),:));
      year_f = deblank(SET_YEAR_F(k2(i),:));
      fprintf80(sprintf('              : %s [%s-%s]',name,year_o,year_f));
      NAMES{i} = name;
    end

     if (length(k2) > 1)
       return;
     elseif (length(k2) == 1)     
       fprintf80(sprintf(['ts_list       : Calling ts_list again '...
			  ' with this name']));
       NAMES = ts_list(NAMES{1});
       return;
     else
       fprintf80(sprintf('ts_list       : No partial matches.  Aborting. '));
       error('');
     end
    
  end
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
  % No match to in_name and no partial match
  if (k == 0) & (length(k2) == 0)
    
    fprintf80(...
	sprintf(...
	    ['ts_list       : Did not find match for data set named '...
	     ' -- %s -- in any file in directories %s'],in_name,...
	    TSDS_DATA_LOCAL_WRITE));

      fprintf80(...
	  sprintf(...
	      ['ts_list       : Returning empty arrays. You may need '...
	       'to update your data base by executing '...
	       'TS_LIST(1).']));
  end
  %error('Aborting due to above problem.');
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
  
  NAMES = [];

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
  if (k == 1)

    if (is_unix == 1)
      slash = '/';
    end
    if (is_unix == 0)
      slash = '\\'; % Octave does not like a lone forward slash.
    end

    if (SET_DIR_NUM(JTEMP(1)) == '0')
      base_dir = TSDS_DATA_LOCAL;
    else
      base_dir = TSDS_DATA_LOCAL_WRITE;
    end

    % Get attributes of all time series in file from
    INFO = ts_dfinfo([base_dir,slash(1),FILENAMES{SET_FILE_NUM(JTEMP(1)),1}]);
    set_no = SET_NUM_INFILE(JTEMP(1));
    % Remove the -metadataonly string in INFO{set_no}.Filename. 
    fname = INFO{set_no}.Filename;
    a     = findstr('-metadataonly',fname);
    fname = [fname(1:a(1)-1),fname(a(1)+13:end)]; % Remove metadataonly string
    
    INFO{set_no}.Filename = fname;
    
    NAMES = INFO{set_no};
    
  end
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
