function [X,Npd,T,Ng] = ts_slice(NAMES_in,UTo,UTf,dTforce)
%TS_SLICE Get time series data in a Universal Time range
%
%   [X,Npd,T] = TS_SLICE(Names) Returns all available data for time series
%   with names specified in string Names in X.  
%
%   Npd is the number of data values per day.  Names can be a string matrix
%   with rows of time series names or 1-D cell array with elements of time
%   series name.
%
%   If necessary, data are re-sampled to the lowest sampling period by
%   repeating values so that T is the union of the available times for all time
%   series requested.  
%
%   [X,Npd,T] = TS_SLICE(Names,YMD_o,YMD_f) Returns data from the first
%   time interval of the day given by YMD_o = [Year, Month, Day] until the
%   last time interval of the day given by YMD_f for all time series with
%   label(s) string (or cell array) Names.
%  
%   If not specified, YMD_f is the last time interval of the year returned
%   by DATE.  If necessary, data are re-sampled to the lowest sampling
%   period by repeating values.  T is the time stamp of the data.
%
%   [X,Npd,T] = TS_SLICE(Names,YMD_o,YMD_f,dTforce) with dTforce = 'min',
%   'hour', or 'day' forces the output to be at that sampling rate by either
%   re-sampling or averaging.  Any averaging is done with BLOCK_MEAN_NONFLAG
%   with COND = 1.
%
%   If YMD_o and/or YMD_f = [], default values are used. 
%
%   [X,Npd,T,Ngood] = TS_SLICE(...) returns the number of non-flagged
%   elements used for any block averaging.  NaNs are inserted in elements of
%   columns where no averaging was performed.
%   
%   See also: TS_SLICE_DEMO, TS_LIST, TS_GET.
  
% R.S. Weigel 12/02/2004

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
if (~iscell(NAMES_in))
  if (size(NAMES_in,1) > 1)
    NAMES = cellstr(NAMES_in);
  else
    NAMES = {NAMES_in};
  end
else
  NAMES = NAMES_in;
end

if (length(NAMES)>1)
  plural = 's';
else
  plural = '';
end

fprintf('\n');

fprintf80(sprintf(['ts_slice      : Start processing '...
		   'request for %d slice%s'],...
		  length(NAMES),plural));
if (nargin > 1)
fprintf80(sprintf(['ts_slice      :  in time range UTo = [%d,%02d,%02d], ',...
		  'UTf = [%d,%02d,%02d]'],...
		  UTo(1),UTo(2),UTo(3),...
		  UTf(1),UTf(2),UTf(3)));
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
M = 1;
for i = 1:length(NAMES)

  I = ts_list(NAMES{i});
  
  if (isempty(I)) | ~isstruct(I)
    X = [];
    T = [];
    fprintf80(...
	['ts_slice      : Exact match to requested time series not found.'...
	      ' Abort.']);
    error('');
    return;
  end
  Ic{i} = I;
  L(i)  = I.Size(1);

  if (length(I.Size) > 1)
    if (I.Size(2) > 1)
      fprintf80('ts_slice      : 2-D time series in list.\n');
      M = I.Size(2);
    end
  end

  if (isstr(I.PPD))
    I.PPD = str2num(I.PPD);
  end
  if (I.PPD == 1)
    Npd(i)   = 1;
    dT(i)    = 1440;
    INTERVAL{i} = 'day';
  end
  if (I.PPD == 24)
    Npd(i)   = 24;
    dT(i)    = 60;
    INTERVAL{i} = 'hour';
  end      
  if (I.PPD == 1440)
    Npd(i)   = 1440;
    dT(i)    = 1;
    INTERVAL{i} = 'min';
  end      

   Yo(i) = str2num(I.Year_o);
   Yf(i) = str2num(I.Year_f);;    
  
   FLAGVALUE(i) = str2num(I.FLAG);    

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
if (nargin < 2)
  UTo = [min(Yo),1,1];
  fprintf('ts_slice      : No UTo given.  Using [%d,%d,%d]\n',...
	  UTo(1),UTo(2),UTo(3));fflush(stdout);
end
if (nargin < 3)
  TheYear = date;
  TheYear = str2num(TheYear(8:end));
  UTf = [TheYear,12,31];
  fprintf('ts_slice      : No UTf given.  Using [%d,%d,%d]\n',...
	  UTf(1),UTf(2),UTf(3));fflush(stdout);
end
if (isempty(UTo))
  UTo = [min(Yo),1,1];
  fprintf('ts_slice     : No UTo given.  Using [%d,%d,%d]\n',...
	  UTo(1),UTo(2),UTo(3));fflush(stdout);
end
if (isempty(UTf))
  TheYear = date;
  TheYear = str2num(TheYear(8:end));
  UTf = [TheYear,12,31];
  fprintf('ts_slice     : No UTf given.  Using [%d,%d,%d]\n',...
	  UTf(1),UTf(2),UTf(3));fflush(stdout);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
if (min(Yo) > UTo(1))
  warn_str = sprintf(['ts_slice      : Year in UTo is less than Year_o of ',...
		      'any requested data set.  Fill values will be used.']);
  fprintf80(warn_str);
end

if (0)
if (max(Yf) < UTo(1))
  warn_str = sprintf(['ts_slice     : Year of UTf is less than Year_o ', ...
		      'of any requested data set.  Returning empty arrays.']);
  fprintf('\n');
  warning(warn_str);
  X  = [];
  T  = [];
  return;
end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
[maxNpd,Imax] = max(Npd); % max Number per day in input list
[minNpd,Imin] = min(Npd); % min Number per day in input list

if (nargin > 3)
  if (dTforce(1:3) == 'min')
    wantedNpd = 1440;
  end
  if (dTforce(1:3) == 'hou')
    wantedNpd = 24;
  end
  if (dTforce(1:3) == 'day')
    wantedNpd = 1;
  end
else
  wantedNpd = maxNpd;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% Grab requested days
X = [];
for i = 1:length(NAMES)

  padl = 0;
  padr = 0;

  delta = day_number(UTo(1:3)) - day_number([Yo(i) 1 1]);
  if (delta < 0) % Requested day_o lower than available for given time series
    padl = -delta;
    fprintf('ts_slice      : Padding on left: %d days\n',padl);
    a    = 1;
    UTo_use  = [Yo(i) 1 1];
  else
    UTo_use  = UTo;
  end

  delta = day_number(UTf(1:3)) - day_number([Yf(i) 12 31]);
  if (delta > 0) % Requested day_f higher than available for given time series
    padr = delta;
    fprintf('ts_slice      : Padding on right: %d days\n',padr);
    UTf_use = [Yf(i) 12 31];
  else
    UTf_use  = UTf;
  end

  a = 1 + day_number(UTo_use(1:3)) - day_number([Yo(i) 1 1]);
  b = a + day_number(UTf_use(1:3)) - day_number(UTo_use(1:3));

  a      = (a-1)*Npd(i)+1;
  b      = b*Npd(i);
  padr   = padr*Npd(i);
  padl   = padl*Npd(i);
  Nread  = b-a+1;

  if (Nread > 0)
    Index  = {[a 1],[1 1],[Nread 1]};
    [I,x]  = ts_get(NAMES{i},Index);
    x      = [FLAGVALUE(i)*ones(padl,M) ; x ; FLAGVALUE(i)*ones(padr,M)];
  else
    fprintf80(sprintf(['ts_slice      : '...
		       'No data for %s in requested time range.'...
		       ' Returning series with all FLAG values'],NAMES{i}));
    NF = 1+day_number(UTf)-day_number(UTo);
    x = FLAGVALUE(i)*ones(Npd(i)*NF,M);
  end
  
  fprintf80(...
      sprintf(['ts_slice      : Data available in range '...
	       'UTo = [%d,1,1], UTf = [%d,12,31]\n'],...
	      Yo(i),Yf(i)));

  str = sprintf(['ts_slice      : '...
		 'Data is on a 1-%s time grid in file.'],INTERVAL{i});
  fprintf80(str);

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % resample
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  if (wantedNpd > Npd(i)) 
    f = round(wantedNpd/Npd(i));
    if (f > 1)
      fprintf80(sprintf(['ts_slice      '...
			 ' : Resampling %s by factor of %d'],NAMES{i},f));
      for i = 1:size(x,2)
	tmp = repmat(x(:,i)',f,1);
	tmpx(:,i) = tmp(:);
      end
      x = tmpx ; clear tmpx tmp;
      if (nargout > 2)    
	ng = repmat(NaN,length(x),M);
      end
    end
  end
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % average
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  if (wantedNpd < Npd(i))
    f = round(Npd(i)/wantedNpd);
    if (f > 1)
      fprintf80(...
	  sprintf(['ts_slice      : Block averaging %s ' ...
		   'to a %d time step window'],NAMES{i},f));
      if (nargout < 3)
	x = block_mean_nonflag(x,f,FLAGVALUE(i),1);
      else
	[x,ng] = block_mean_nonflag(x,f,FLAGVALUE(i),1);
      end
    end
    if (wantedNpd == Npd(i)) 
      ng = repmat(NaN,length(x),M);
    end
  end
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  X = [X, x];
  
  if (nargout > 3)
    Ng(:,i) = Ng;
  end
  
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
if (nargout > 2)

  if (nargin > 3)
    INTERVAL_Imax = dTforce;
  else
    INTERVAL_Imax = INTERVAL{Imax};
  end  

  if (wantedNpd > 1)
    T = ymd([0:size(X,1)-1]',UTo(1),INTERVAL_Imax);
  else
    T = ymd([1:size(X,1)]'  ,UTo(1),INTERVAL_Imax);
  end

end

if (nargin > 1)
  Npd = round(1440/min(dT));
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

fprintf('ts_slice      : Finished processing request\n');
